@@ -44,6 +44,8 @@
 
 #include "qbasicatomic.h"
 
+#include <qtcore_tracepoints_p.h>
+
 #include <limits>
 
 QT_BEGIN_NAMESPACE
@@ -202,6 +204,12 @@ QT_BEGIN_NAMESPACE
     \value Scroll                           The object needs to scroll to the supplied position (QScrollEvent).
     \value Shortcut                         Key press in child for shortcut key handling (QShortcutEvent).
     \value ShortcutOverride                 Key press in child, for overriding shortcut key handling (QKeyEvent).
+                                            When a shortcut is about to trigger, \c ShortcutOverride
+                                            is sent to the active window. This allows clients (e.g. widgets)
+                                            to signal that they will handle the shortcut themselves, by
+                                            accepting the event. If the shortcut override is accepted, the
+                                            event is delivered as a normal key press to the focus widget.
+                                            Otherwise, it triggers the shortcut action, if one exists.
     \value Show                             Widget was shown on screen (QShowEvent).
     \value ShowToParent                     A child widget has been shown.
     \value SockAct                          Socket activated, used to implement QSocketNotifier.
@@ -288,7 +296,9 @@ QT_BEGIN_NAMESPACE
 */
 QEvent::QEvent(Type type)
     : d(0), t(type), posted(false), spont(false), m_accept(true)
-{}
+{
+    Q_TRACE(QEvent_ctor, this, t);
+}
 
 /*!
     \internal
@@ -301,6 +311,7 @@ QEvent::QEvent(const QEvent &other)
     : d(other.d), t(other.t), posted(other.posted), spont(other.spont),
       m_accept(other.m_accept)
 {
+    Q_TRACE(QEvent_ctor, this, t);
     // if QEventPrivate becomes available, make sure to implement a
     // virtual QEventPrivate *clone() const; function so we can copy here
     Q_ASSERT_X(!d, "QEvent", "Impossible, this can't happen: QEventPrivate isn't defined anywhere");
@@ -333,6 +344,7 @@ QEvent &QEvent::operator=(const QEvent &other)
 
 QEvent::~QEvent()
 {
+    Q_TRACE(QEvent_dtor, this, t);
     if (posted && QCoreApplication::instance())
         QCoreApplicationPrivate::removePostedEvent(this);
     Q_ASSERT_X(!d, "QEvent", "Impossible, this can't happen: QEventPrivate isn't defined anywhere");
@@ -409,10 +421,10 @@ struct QBasicAtomicBitField {
     QBasicAtomicInteger<uint> next;
     QBasicAtomicInteger<uint> data[NumInts];
 
-    bool allocateSpecific(int which) Q_DECL_NOTHROW
+    bool allocateSpecific(int which) noexcept
     {
         QBasicAtomicInteger<uint> &entry = data[which / BitsPerInt];
-        const uint old = entry.load();
+        const uint old = entry.loadRelaxed();
         const uint bit = 1U << (which % BitsPerInt);
         return !(old & bit) // wasn't taken
             && entry.testAndSetRelaxed(old, old | bit); // still wasn't taken
@@ -425,7 +437,7 @@ struct QBasicAtomicBitField {
         // loop.
     }
 
-    int allocateNext() Q_DECL_NOTHROW
+    int allocateNext() noexcept
     {
         // Unroll loop to iterate over ints, then bits? Would save
         // potentially a lot of cmpxchgs, because we can scan the
@@ -433,10 +445,10 @@ struct QBasicAtomicBitField {
 
         // Then again, this should never execute many iterations, so
         // leave like this for now:
-        for (uint i = next.load(); i < NumBits; ++i) {
+        for (uint i = next.loadRelaxed(); i < NumBits; ++i) {
             if (allocateSpecific(i)) {
                 // remember next (possibly) free id:
-                const uint oldNext = next.load();
+                const uint oldNext = next.loadRelaxed();
                 next.testAndSetRelaxed(oldNext, qMax(i + 1, oldNext));
                 return i;
             }
@@ -451,7 +463,7 @@ typedef QBasicAtomicBitField<QEvent::MaxUser - QEvent::User + 1> UserEventTypeRe
 
 static UserEventTypeRegistry userEventTypeRegistry;
 
-static inline int registerEventTypeZeroBased(int id) Q_DECL_NOTHROW
+static inline int registerEventTypeZeroBased(int id) noexcept
 {
     // if the type hint hasn't been registered yet, take it:
     if (id < UserEventTypeRegistry::NumBits && id >= 0 && userEventTypeRegistry.allocateSpecific(id))
@@ -474,7 +486,7 @@ static inline int registerEventTypeZeroBased(int id) Q_DECL_NOTHROW
     Returns -1 if all available values are already taken or the
     program is shutting down.
 */
-int QEvent::registerEventType(int hint) Q_DECL_NOTHROW
+int QEvent::registerEventType(int hint) noexcept
 {
     const int result = registerEventTypeZeroBased(QEvent::MaxUser - hint);
     return result < 0 ? -1 : QEvent::MaxUser - result ;
