@@ -45,6 +45,7 @@
 
 #include <QtCore/QThreadStorage>
 #include <QtCore/QThread>
+#include <QtCore/private/qlocking_p.h>
 
 #include <QtGui/private/qguiapplication_p.h>
 #include <QtGui/private/qopengl_p.h>
@@ -313,7 +314,7 @@ QOpenGLContext *qt_gl_global_share_context()
 
     \section1 Context Resource Sharing
 
-    Resources, such as framebuffer objects, textures, and vertex buffer objects
+    Resources such as textures and vertex buffer objects
     can be shared between contexts.  Use setShareContext() before calling
     create() to specify that the contexts should share these resources.
     QOpenGLContext internally keeps track of a QOpenGLContextGroup object which
@@ -414,15 +415,14 @@ int QOpenGLContextPrivate::maxTextureSize()
 
 /*!
     Returns the last context which called makeCurrent in the current thread,
-    or 0, if no context is current.
+    or \nullptr, if no context is current.
 */
 QOpenGLContext* QOpenGLContext::currentContext()
 {
     QGuiGLThreadContext *threadContext = qwindow_context_storage()->localData();
-    if (threadContext) {
+    if (threadContext)
         return threadContext->context;
-    }
-    return 0;
+    return nullptr;
 }
 
 /*!
@@ -612,8 +612,8 @@ bool QOpenGLContext::create()
     d->platformGLContext = QGuiApplicationPrivate::platformIntegration()->createPlatformOpenGLContext(this);
     if (!d->platformGLContext)
         return false;
-    d->platformGLContext->initialize();
     d->platformGLContext->setContext(this);
+    d->platformGLContext->initialize();
     if (!d->platformGLContext->isSharing())
         d->shareContext = 0;
     d->shareGroup = d->shareContext ? d->shareContext->shareGroup() : new QOpenGLContextGroup;
@@ -788,7 +788,7 @@ QOpenGLExtraFunctions *QOpenGLContext::extraFunctions() const
     to the non-template function.
 
     Note that requests for function objects of other versions or profiles can fail and
-    in doing so will return a null pointer. Situations in which creation of the functions
+    in doing so will return \nullptr. Situations in which creation of the functions
     object can fail are if the request cannot be satisfied due to asking for functions
     that are not in the version or profile of this context. For example:
 
@@ -945,7 +945,7 @@ GLuint QOpenGLContext::defaultFramebufferObject() const
 
     Avoid calling this function from a different thread than the one the
     QOpenGLContext instance lives in. If you wish to use QOpenGLContext from a
-    different thread you should first call make sure it's not current in the
+    different thread you should first make sure it's not current in the
     current thread, by calling doneCurrent() if necessary. Then call
     moveToThread(otherThread) before using it in the other thread.
 
@@ -981,59 +981,59 @@ bool QOpenGLContext::makeCurrent(QSurface *surface)
         return false;
     }
 
-    QOpenGLContext *previous = QOpenGLContextPrivate::setCurrentContext(this);
-
-    if (d->platformGLContext->makeCurrent(surface->surfaceHandle())) {
-        static bool needsWorkaroundSet = false;
-        static bool needsWorkaround = false;
-
-        if (!needsWorkaroundSet) {
-            QByteArray env;
-#ifdef Q_OS_ANDROID
-            env = qgetenv(QByteArrayLiteral("QT_ANDROID_DISABLE_GLYPH_CACHE_WORKAROUND"));
-            needsWorkaround = env.isEmpty() || env == QByteArrayLiteral("0") || env == QByteArrayLiteral("false");
-#endif
-            env = qgetenv(QByteArrayLiteral("QT_ENABLE_GLYPH_CACHE_WORKAROUND"));
-            if (env == QByteArrayLiteral("1") || env == QByteArrayLiteral("true"))
-                needsWorkaround = true;
-
-            if (!needsWorkaround) {
-                const char *rendererString = reinterpret_cast<const char *>(functions()->glGetString(GL_RENDERER));
-                if (rendererString)
-                    needsWorkaround =
-                            qstrncmp(rendererString, "Mali-4xx", 6) == 0 // Mali-400, Mali-450
-                            || qstrcmp(rendererString, "Mali-T880") == 0
-                            || qstrncmp(rendererString, "Adreno (TM) 2xx", 13) == 0 // Adreno 200, 203, 205
-                            || qstrncmp(rendererString, "Adreno 2xx", 8) == 0 // Same as above but without the '(TM)'
-                            || qstrncmp(rendererString, "Adreno (TM) 30x", 14) == 0 // Adreno 302, 305
-                            || qstrncmp(rendererString, "Adreno 30x", 9) == 0 // Same as above but without the '(TM)'
-                            || qstrncmp(rendererString, "Adreno (TM) 4xx", 13) == 0 // Adreno 405, 418, 420, 430
-                            || qstrncmp(rendererString, "Adreno 4xx", 8) == 0 // Same as above but without the '(TM)'
-                            || qstrcmp(rendererString, "GC800 core") == 0
-                            || qstrcmp(rendererString, "GC1000 core") == 0
-                            || strstr(rendererString, "GC2000") != 0
-                            || qstrcmp(rendererString, "Immersion.16") == 0;
-            }
-            needsWorkaroundSet = true;
-        }
-
-        if (needsWorkaround)
-            d->workaround_brokenFBOReadBack = true;
-
-        d->surface = surface;
-
-        d->shareGroup->d_func()->deletePendingResources(this);
+    if (!d->platformGLContext->makeCurrent(surface->surfaceHandle()))
+        return false;
 
+    QOpenGLContextPrivate::setCurrentContext(this);
 #ifndef QT_NO_DEBUG
-        QOpenGLContextPrivate::toggleMakeCurrentTracker(this, true);
+    QOpenGLContextPrivate::toggleMakeCurrentTracker(this, true);
 #endif
 
-        return true;
+    d->surface = surface;
+
+    static bool needsWorkaroundSet = false;
+    static bool needsWorkaround = false;
+
+    if (!needsWorkaroundSet) {
+        QByteArray env;
+#ifdef Q_OS_ANDROID
+        env = qgetenv(QByteArrayLiteral("QT_ANDROID_DISABLE_GLYPH_CACHE_WORKAROUND"));
+        needsWorkaround = env.isEmpty() || env == QByteArrayLiteral("0") || env == QByteArrayLiteral("false");
+#endif
+        env = qgetenv(QByteArrayLiteral("QT_ENABLE_GLYPH_CACHE_WORKAROUND"));
+        if (env == QByteArrayLiteral("1") || env == QByteArrayLiteral("true"))
+            needsWorkaround = true;
+
+        if (!needsWorkaround) {
+            const char *rendererString = reinterpret_cast<const char *>(functions()->glGetString(GL_RENDERER));
+            if (rendererString)
+                needsWorkaround =
+                        qstrncmp(rendererString, "Mali-4xx", 6) == 0 // Mali-400, Mali-450
+                        || qstrcmp(rendererString, "Mali-T880") == 0
+                        || qstrncmp(rendererString, "Adreno (TM) 2xx", 13) == 0 // Adreno 200, 203, 205
+                        || qstrncmp(rendererString, "Adreno 2xx", 8) == 0 // Same as above but without the '(TM)'
+                        || qstrncmp(rendererString, "Adreno (TM) 3xx", 13) == 0 // Adreno 302, 305, 320, 330
+                        || qstrncmp(rendererString, "Adreno 3xx", 8) == 0 // Same as above but without the '(TM)'
+                        || qstrncmp(rendererString, "Adreno (TM) 4xx", 13) == 0 // Adreno 405, 418, 420, 430
+                        || qstrncmp(rendererString, "Adreno 4xx", 8) == 0 // Same as above but without the '(TM)'
+                        || qstrncmp(rendererString, "Adreno (TM) 5xx", 13) == 0 // Adreno 505, 506, 510, 530, 540
+                        || qstrncmp(rendererString, "Adreno 5xx", 8) == 0 // Same as above but without the '(TM)'
+                        || qstrncmp(rendererString, "Adreno (TM) 6xx", 13) == 0 // Adreno 610, 620, 630
+                        || qstrncmp(rendererString, "Adreno 6xx", 8) == 0 // Same as above but without the '(TM)'
+                        || qstrcmp(rendererString, "GC800 core") == 0
+                        || qstrcmp(rendererString, "GC1000 core") == 0
+                        || strstr(rendererString, "GC2000") != 0
+                        || qstrcmp(rendererString, "Immersion.16") == 0;
+        }
+        needsWorkaroundSet = true;
     }
 
-    QOpenGLContextPrivate::setCurrentContext(previous);
+    if (needsWorkaround)
+        d->workaround_brokenFBOReadBack = true;
 
-    return false;
+    d->shareGroup->d_func()->deletePendingResources(this);
+
+    return true;
 }
 
 /*!
@@ -1074,7 +1074,8 @@ QSurface *QOpenGLContext::surface() const
     Swap the back and front buffers of \a surface.
 
     Call this to finish a frame of OpenGL rendering, and make sure to
-    call makeCurrent() again before you begin a new frame.
+    call makeCurrent() again before issuing any further OpenGL commands,
+    for example as part of a new frame.
 */
 void QOpenGLContext::swapBuffers(QSurface *surface)
 {
@@ -1114,7 +1115,7 @@ void QOpenGLContext::swapBuffers(QSurface *surface)
 /*!
     Resolves the function pointer to an OpenGL extension function, identified by \a procName
 
-    Returns 0 if no such function can be found.
+    Returns \nullptr if no such function can be found.
 */
 QFunctionPointer QOpenGLContext::getProcAddress(const QByteArray &procName) const
 {
@@ -1232,7 +1233,8 @@ void QOpenGLContext::deleteQGLContext()
   Returns the platform-specific handle for the OpenGL implementation that
   is currently in use. (for example, a HMODULE on Windows)
 
-  On platforms that do not use dynamic GL switch the return value is null.
+  On platforms that do not use dynamic GL switching, the return value
+  is \nullptr.
 
   The library might be GL-only, meaning that windowing system interface
   functions (for example EGL) may live in another, separate library.
@@ -1326,7 +1328,7 @@ bool QOpenGLContext::supportsThreadedOpenGL()
     \since 5.5
 
     Returns the application-wide shared OpenGL context, if present.
-    Otherwise, returns a null pointer.
+    Otherwise, returns \nullptr.
 
     This is useful if you need to upload OpenGL objects (buffers, textures,
     etc.) before creating or showing a QOpenGLWidget or QQuickWidget.
@@ -1441,7 +1443,7 @@ QOpenGLContextGroup *QOpenGLContextGroup::currentContextGroup()
 
 void QOpenGLContextGroupPrivate::addContext(QOpenGLContext *ctx)
 {
-    QMutexLocker locker(&m_mutex);
+    const auto locker = qt_scoped_lock(m_mutex);
     m_refs.ref();
     m_shares << ctx;
 }
@@ -1453,7 +1455,7 @@ void QOpenGLContextGroupPrivate::removeContext(QOpenGLContext *ctx)
     bool deleteObject = false;
 
     {
-        QMutexLocker locker(&m_mutex);
+        const auto locker = qt_scoped_lock(m_mutex);
         m_shares.removeOne(ctx);
 
         if (ctx == m_context && !m_shares.isEmpty())
@@ -1501,7 +1503,7 @@ void QOpenGLContextGroupPrivate::cleanup()
 
 void QOpenGLContextGroupPrivate::deletePendingResources(QOpenGLContext *ctx)
 {
-    QMutexLocker locker(&m_mutex);
+    const auto locker = qt_scoped_lock(m_mutex);
 
     const QList<QOpenGLSharedResource *> pending = m_pendingDeletion;
     m_pendingDeletion.clear();
@@ -1542,7 +1544,7 @@ void QOpenGLContextGroupPrivate::deletePendingResources(QOpenGLContext *ctx)
 QOpenGLSharedResource::QOpenGLSharedResource(QOpenGLContextGroup *group)
     : m_group(group)
 {
-    QMutexLocker locker(&m_group->d_func()->m_mutex);
+    const auto locker = qt_scoped_lock(m_group->d_func()->m_mutex);
     m_group->d_func()->m_sharedResources << this;
 }
 
@@ -1558,7 +1560,7 @@ void QOpenGLSharedResource::free()
         return;
     }
 
-    QMutexLocker locker(&m_group->d_func()->m_mutex);
+    const auto locker = qt_scoped_lock(m_group->d_func()->m_mutex);
     m_group->d_func()->m_sharedResources.removeOne(this);
     m_group->d_func()->m_pendingDeletion << this;
 
@@ -1606,8 +1608,7 @@ void QOpenGLSharedResourceGuard::freeResource(QOpenGLContext *context)
     QOpenGLMultiGroupSharedResource instance.
 */
 QOpenGLMultiGroupSharedResource::QOpenGLMultiGroupSharedResource()
-    : active(0),
-      m_mutex(QMutex::Recursive)
+    : active(0)
 {
 #ifdef QT_GL_CONTEXT_RESOURCE_DEBUG
     qDebug("Creating context group resource object %p.", this);
@@ -1630,7 +1631,7 @@ QOpenGLMultiGroupSharedResource::~QOpenGLMultiGroupSharedResource()
         active.deref();
     }
 #ifndef QT_NO_DEBUG
-    if (active.load() != 0) {
+    if (active.loadRelaxed() != 0) {
         qWarning("QtGui: Resources are still available at program shutdown.\n"
                  "          This is possibly caused by a leaked QOpenGLWidget, \n"
                  "          QOpenGLFramebufferObject or QOpenGLPixelBuffer.");
