@@ -36,9 +36,6 @@
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
-#if 0 // Used to be included in Qt4 for Q_WS_MAC
-# include <private/qcore_mac_p.h>
-#endif
 
 #include <QtWidgets/private/qtwidgetsglobal_p.h>
 
@@ -56,17 +53,16 @@
 #endif
 #include <qtextdocument.h>
 #include <qdebug.h>
+#include <qpa/qplatformscreen.h>
+#include <qpa/qplatformcursor.h>
 #include <private/qstylesheetstyle_p.h>
 
 #ifndef QT_NO_TOOLTIP
 #include <qlabel.h>
+#include <QtWidgets/private/qlabel_p.h>
+#include <QtGui/private/qhighdpiscaling_p.h>
 #include <qtooltip.h>
 
-#if 0 // Used to be included in Qt4 for Q_WS_MAC
-# include <private/qcore_mac_p.h>
-#include <private/qt_cocoa_helpers_mac_p.h>
-#endif
-
 QT_BEGIN_NAMESPACE
 
 /*!
@@ -127,6 +123,7 @@ public:
     ~QTipLabel();
     static QTipLabel *instance;
 
+    void adjustTooltipScreen(const QPoint &pos);
     void updateSize(const QPoint &pos);
 
     bool eventFilter(QObject *, QEvent *) override;
@@ -214,7 +211,6 @@ void QTipLabel::reuseTip(const QString &text, int msecDisplayTime, const QPoint
     }
 #endif
 
-    setWordWrap(true);
     setText(text);
     updateSize(pos);
     restartExpireTimer(msecDisplayTime);
@@ -222,24 +218,28 @@ void QTipLabel::reuseTip(const QString &text, int msecDisplayTime, const QPoint
 
 void  QTipLabel::updateSize(const QPoint &pos)
 {
+#ifndef Q_OS_WINRT
+    // ### The code below does not always work well on WinRT
+    // (e.g COIN fails an auto test - tst_QToolTip::qtbug64550_stylesheet - QTBUG-72652)
+    d_func()->setScreenForPoint(pos);
+#endif
+    // Ensure that we get correct sizeHints by placing this window on the right screen.
     QFontMetrics fm(font());
     QSize extra(1, 0);
     // Make it look good with the default ToolTip font on Mac, which has a small descent.
     if (fm.descent() == 2 && fm.ascent() >= 11)
         ++extra.rheight();
+    setWordWrap(Qt::mightBeRichText(text()));
     QSize sh = sizeHint();
-    if (wordWrap()) {
-        QScreen *screen = QGuiApplication::screenAt(pos);
-        if (!screen)
-            screen = QGuiApplication::primaryScreen();
-        if (screen) {
-            const qreal screenWidth = screen->geometry().width();
-            if (sh.width() > screenWidth) {
-                // Try to use widely accepted 75chars max length or 80% of the screen width else.
-                // See https://en.wikipedia.org/wiki/Line_length
-                sh.setWidth(qMin(fm.averageCharWidth() * 75, static_cast<int>(screenWidth * .8)));
-                sh.setHeight(heightForWidth(sh.width()));
-            }
+    // ### When the above WinRT code is fixed, windowhandle should be used to find the screen.
+    QScreen *screen = QGuiApplication::screenAt(pos);
+    if (!screen)
+        screen = QGuiApplication::primaryScreen();
+    if (screen) {
+        const qreal screenWidth = screen->geometry().width();
+        if (!wordWrap() && sh.width() > screenWidth) {
+            setWordWrap(true);
+            sh = sizeHint();
         }
     }
     resize(sh + extra);
@@ -312,20 +312,7 @@ void QTipLabel::timerEvent(QTimerEvent *e)
         || e->timerId() == expireTimer.timerId()){
         hideTimer.stop();
         expireTimer.stop();
-#if 0 /* Used to be included in Qt4 for Q_WS_MAC */ && QT_CONFIG(effects)
-        if (QApplication::isEffectEnabled(Qt::UI_FadeTooltip)){
-            // Fade out tip on mac (makes it invisible).
-            // The tip will not be deleted until a new tip is shown.
-
-                        // DRSWAT - Cocoa
-                        macWindowFade(qt_mac_window_for(this));
-            QTipLabel::instance->fadingOut = true; // will never be false again.
-        }
-        else
-            hideTipImmediately();
-#else
         hideTipImmediately();
-#endif
     }
 }
 
@@ -414,42 +401,34 @@ void QTipLabel::placeTip(const QPoint &pos, QWidget *w)
     }
 #endif //QT_NO_STYLE_STYLESHEET
 
-
-#if 0 // Used to be included in Qt4 for Q_WS_MAC
-    // When in full screen mode, there is no Dock nor Menu so we can use
-    // the whole screen for displaying the tooltip. However when not in
-    // full screen mode we need to save space for the dock, so we use
-    // availableGeometry instead.
-    extern bool qt_mac_app_fullscreen; //qapplication_mac.mm
-    QRect screen;
-    if(qt_mac_app_fullscreen)
-        screen = QDesktopWidgetPrivate::screenGeometry(getTipScreen(pos, w));
-    else
-        screen = QDesktopWidgetPrivate::availableGeometry(getTipScreen(pos, w));
-#else
-    QRect screen = QDesktopWidgetPrivate::screenGeometry(getTipScreen(pos, w));
-#endif
-
     QPoint p = pos;
-    p += QPoint(2,
-#if 0 // Used to be included in Qt4 for Q_WS_WIN
-                21
-#else
-                16
-#endif
-        );
-    if (p.x() + this->width() > screen.x() + screen.width())
+    const QScreen *screen = QGuiApplication::screens().value(getTipScreen(pos, w),
+                                                             QGuiApplication::primaryScreen());
+    // a QScreen's handle *should* never be null, so this is a bit paranoid
+    if (const QPlatformScreen *platformScreen = screen ? screen->handle() : nullptr) {
+        const QSize cursorSize = QHighDpi::fromNativePixels(platformScreen->cursor()->size(),
+                                                            platformScreen);
+        QPoint offset(2, cursorSize.height());
+        // assuming an arrow shape, we can just move to the side for very large cursors
+        if (cursorSize.height() > 2 * this->height())
+            offset = QPoint(cursorSize.width() / 2, 0);
+
+        p += offset;
+
+        QRect screenRect = screen->geometry();
+        if (p.x() + this->width() > screenRect.x() + screenRect.width())
         p.rx() -= 4 + this->width();
-    if (p.y() + this->height() > screen.y() + screen.height())
+        if (p.y() + this->height() > screenRect.y() + screenRect.height())
         p.ry() -= 24 + this->height();
-    if (p.y() < screen.y())
-        p.setY(screen.y());
-    if (p.x() + this->width() > screen.x() + screen.width())
-        p.setX(screen.x() + screen.width() - this->width());
-    if (p.x() < screen.x())
-        p.setX(screen.x());
-    if (p.y() + this->height() > screen.y() + screen.height())
-        p.setY(screen.y() + screen.height() - this->height());
+        if (p.y() < screenRect.y())
+            p.setY(screenRect.y());
+        if (p.x() + this->width() > screenRect.x() + screenRect.width())
+            p.setX(screenRect.x() + screenRect.width() - this->width());
+        if (p.x() < screenRect.x())
+            p.setX(screenRect.x());
+        if (p.y() + this->height() > screenRect.y() + screenRect.height())
+            p.setY(screenRect.y() + screenRect.height() - this->height());
+    }
     this->move(p);
 }
 
@@ -477,8 +456,8 @@ bool QTipLabel::tipChanged(const QPoint &pos, const QString &text, QObject *o)
 
     The \a rect is in the coordinates of the widget you specify with
     \a w. If the \a rect is not empty you must specify a widget.
-    Otherwise this argument can be 0 but it is used to determine the
-    appropriate screen on multi-head systems.
+    Otherwise this argument can be \nullptr but it is used to
+    determine the appropriate screen on multi-head systems.
 
     If \a text is empty the tool tip is hidden. If the text is the
     same as the currently shown tooltip, the tip will \e not move.
@@ -536,7 +515,7 @@ QT_WARNING_POP
         QTipLabel::instance->setObjectName(QLatin1String("qtooltip_label"));
 
 
-#if QT_CONFIG(effects) && !0 /* Used to be included in Qt4 for Q_WS_MAC */
+#if QT_CONFIG(effects)
         if (QApplication::isEffectEnabled(Qt::UI_FadeTooltip))
             qFadeEffect(QTipLabel::instance);
         else if (QApplication::isEffectEnabled(Qt::UI_AnimateTooltip))
